/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.swaggercombine;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.redhat.thermostat.common.yaml.YamlToJson;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Utility class to build a SwaggerCombine object
 */
public class SwaggerCombineContext {

    public enum OutputFormat { NONE, YAML, JSON };

    private OutputFormat fmt = OutputFormat.NONE;
    private boolean pretty = false;
    private boolean quiet = false;
    private boolean useTemplate = false;
    private boolean lint = true;
    private boolean printUsage = false;
    private final List<JsonObject> apiList = new ArrayList<>();

    // potential extra args to allow for when building arg array
    private static final int EXTRA_ARG_COUNT = 5;


    public SwaggerCombineContext() {
    }

    /**
     * defint the output format
     * @param fmt one of OutputFormat.YAML | JSON or NONE
     * @return this
     */
    public SwaggerCombineContext produce(OutputFormat fmt) {
        this.fmt = fmt;
        return this;
    }

    /**
     * set pretty printing on or off
     * @param pp true to set pretty printing on
     * @return this
     */
    public SwaggerCombineContext prettyPrint(boolean pp) {
        this.pretty = pp;
        return this;
    }

    /**
     * set quiet mode
     * @param pp troue to set quiet mode (no progress messages)
     * @return this
     */
    public SwaggerCombineContext quiet(boolean pp) {
        this.quiet = pp;
        return this;
    }

    /**
     * set initial document to built in template
     * @param pp true to use built in template as initial document
     *           If not used, the root elements (version, title, etc) of the first document specified are used.
     * @return this
     */
    public SwaggerCombineContext useTemplate(boolean pp) {
        this.useTemplate = pp;
        return this;
    }

    /**
     * run basic tests on input documents
     * @param pp true to run initial tests
     * @return this
     */
    public SwaggerCombineContext lint(boolean pp) {
        this.lint = pp;
        return this;
    }

    /**
     * add an input file (representing a microAPI defined by Swagger)
     * @param fin input micro API definitions
     * @return this
     * @throws IOException if an I/O error occurred
     */
    public SwaggerCombineContext addMicroAPI(File fin) throws IOException {
        final JsonObject json = SwaggerCombine.readSwaggerFile(fin);
        if (json != null) {
            this.getAPIList().add(json);
        }
        return this;
    }

    /**
     * add an input file (representing a microAPI defined by Swagger)
     * @param spec input micro API definitions
     * @param isYaml true if format of input file is YAML, false if gormat is JSON
     * @return this
     * @throws IOException if an I/O error occurred
     */
    public SwaggerCombineContext addMicroAPI(String spec, boolean isYaml) throws IOException {
        final JsonObject json;
        if (isYaml) {
            final YamlToJson y2j = new YamlToJson();
            json = y2j.yamlToJsonObject(spec);
        } else {
            final JsonParser parser = new JsonParser();
            json = parser.parse(new BufferedReader(new StringReader(spec))).getAsJsonObject();
        }
        this.getAPIList().add(json);
        return this;
    }

    /**
     * set quiet mode
     * @param pp troue to set quiet mode (no progress messages)
     * @return this
     */
    public SwaggerCombineContext usage(boolean pp) {
        this.printUsage = pp;
        return this;
    }

    public static int getExtraArgCount() {
        return EXTRA_ARG_COUNT;
    }

    public OutputFormat getFmt() {
        return fmt;
    }

    public boolean isPretty() {
        return pretty;
    }

    public boolean isQuiet() {
        return quiet;
    }

    public boolean isUseTemplate() {
        return useTemplate;
    }

    public boolean isLint() {
        return lint;
    }

    public boolean printUsage() {
        return printUsage;
    }

    public List<JsonObject> getAPIList() {
        return apiList;
    }

}
